use minifb::{InputCallback, Key};
use std::sync::{Arc, Mutex};

use crate::memory::Mem;

pub struct Keyboard {
    memory: Arc<Mutex<Mem>>,
}

impl Keyboard {
    pub fn new(memory: Arc<Mutex<Mem>>) -> Self {
        Self { memory }
    }
}

impl InputCallback for Keyboard {
    fn add_char(&mut self, _uni_char: u32) {}
    fn set_key_state(&mut self, key: Key, _state: bool) {
        let mut row0 = 0;
        let mut row1 = 0;
        let mut row2 = 0;
        let mut row3 = 0;
        let mut row4 = 0;
        let mut row5 = 0;

        match key {
            Key::Escape => row0 ^= 0b1000_0000,
            Key::W => row0 ^= 0b0100_0000,
            Key::E => row0 ^= 0b0010_0000,
            Key::R => row0 ^= 0b0001_0000,
            Key::T => row0 ^= 0b0000_1000,
            Key::U => row0 ^= 0b0000_0100,
            Key::O => row0 ^= 0b0000_0010,
            Key::Backspace => row0 ^= 0b0000_0001,
            Key::Tab => row1 ^= 0b1000_0000,
            Key::Q => row1 ^= 0b0100_0000,
            Key::S => row1 ^= 0b0010_0000,
            Key::G => row1 ^= 0b0001_0000,
            Key::Y => row1 ^= 0b0000_1000,
            Key::I => row1 ^= 0b0000_0100,
            Key::P => row1 ^= 0b0000_0010,
            Key::Enter => row1 ^= 0b0000_0001,
            Key::LeftShift | Key::RightShift => row2 ^= 0b1000_0000,
            Key::D => row2 ^= 0b0100_0000,
            Key::V => row2 ^= 0b0010_0000,
            Key::H => row2 ^= 0b0001_0000,
            Key::K => row2 ^= 0b0000_1000,
            Key::Apostrophe => row2 ^= 0b0000_0100,
            Key::Slash => row2 ^= 0b0000_0010,
            Key::A => row2 ^= 0b0000_0001,
            Key::LeftCtrl | Key::RightCtrl => row3 ^= 0b1000_0000,
            Key::Z => row3 ^= 0b0100_0000,
            Key::F => row3 ^= 0b0010_0000,
            Key::B => row3 ^= 0b0001_0000,
            Key::J => row3 ^= 0b0000_1000,
            Key::L => row3 ^= 0b0000_0100,
            Key::Key2 => row3 ^= 0b0000_0010,
            Key::Key4 => row3 ^= 0b0000_0001,
            Key::LeftAlt | Key::RightAlt => row4 ^= 0b1000_0000,
            Key::X => row4 ^= 0b0100_0000,
            Key::C => row4 ^= 0b0010_0000,
            Key::N => row4 ^= 0b0001_0000,
            Key::M => row4 ^= 0b0000_1000,
            Key::Comma => row4 ^= 0b0000_0100,
            Key::Key1 => row4 ^= 0b0000_0010,
            Key::Key3 => row4 ^= 0b0000_0001,
            Key::LeftSuper => row5 ^= 0b1000_0000,
            Key::Space => row5 ^= 0b0100_0000,
            Key::RightSuper => row5 ^= 0b0010_0000,
            _ => {}
        };

        match &mut self.memory.lock() {
            Ok(memory) => {
                memory.write(0x4400, row0);
                memory.write(0x4401, row1);
                memory.write(0x4402, row2);
                memory.write(0x4403, row3);
                memory.write(0x4404, row4);
                memory.write(0x4405, row5);
            }
            Err(_error) => {
                println!("couldnt get a lock on memory while saving keyboard registers");
            }
        }
    }
}
