use crate::error::MemoryError;
use crate::types::{Byte, Word};
use std::io::{self, Write};
use std::path::PathBuf;
use std::u16;
use std::{fs::File, io::Read};

#[derive(Debug, Clone)]
pub struct Mem {
    pub data: Vec<Byte>,
}

impl Mem {
    pub fn new() -> Self {
        Self {
            data: vec![0; u16::MAX as usize + 1],
        }
    }
    pub fn dump(&self, path: PathBuf) -> io::Result<()> {
        let mut outfile = File::create(path)?;
        outfile.write_all(&self.data)?;
        Ok(())
    }

    pub fn read(&self, address: Word) -> Byte {
        self.data[address as usize]
    }

    pub fn write(&mut self, address: Word, data: Byte) {
        self.data[address as usize] = data;
    }

    pub fn load_rom(&mut self, rom: File) -> Result<(), MemoryError> {
        let bytes = rom.bytes();
        for (address, byte) in bytes.enumerate() {
            match byte {
                Ok(value) => self.write(address as Word + 0x8000, value),
                Err(_) => {
                    println!("Loading rom: couldn't write byte {:#04x}", address);
                    return Err(MemoryError::Unwritable);
                }
            }
        }
        Ok(())
    }
    //pub fn read_from_bin(&mut self, f: File) -> Result<(), MemoryError> {
    //    let bytes = f.bytes();
    //    for (address, byte) in bytes.enumerate() {
    //        match byte {
    //            Ok(value) => self.write(address as Word, value),
    //            Err(_) => {
    //                println!("couldn't write byte {:#04x}", address);
    //                return Err(MemoryError::Unwritable);
    //            }
    //        }
    //    }
    //    Ok(())
    //}
}
