use crate::Mem;
use std::{
    fs::File,
    io::Read,
    path::Path,
    sync::{mpsc::Sender, Arc, Mutex},
    thread::sleep,
    time::Duration,
};

const FG_COLOR: u32 = 0xFFCC00;
const BG_COLOR: u32 = 0x110500;

pub struct Crtc {
    memory: Arc<Mutex<Mem>>,
    buffer: Vec<u32>,
    char_rom: Vec<u8>,
    interrupt: Sender<bool>,
    window: Sender<Vec<u32>>,
}

pub fn get_char_bin<P>(char_rom: Option<P>) -> Vec<u8>
where
    P: AsRef<Path>,
{
    match char_rom {
        Some(path) => {
            let mut file = File::open(path).unwrap();
            let mut bin = vec![0; 0x8000];
            file.read_exact(&mut bin).unwrap();
            // println!("reading char rom");
            bin
        }
        None => include_bytes!("./roms/cozette.rom").to_vec(),
    }
}

impl Crtc {
    pub fn new<P>(
        memory: Arc<Mutex<Mem>>,
        char_rom: Option<P>,
        interrupt: Sender<bool>,
        window: Sender<Vec<u32>>,
    ) -> Self
    where
        P: AsRef<Path>,
    {
        let char_rom = get_char_bin(char_rom);
        Self {
            memory,
            buffer: vec![0; 512 * 380],
            window,
            char_rom,
            interrupt,
        }
    }
    fn draw(&mut self) {
        let memory = match self.memory.lock() {
            Ok(memory) => memory,
            Err(_) => {
                println!("Couldn't acquire read on shared memory in video thread");
                return;
            }
        };

        // the rest of this function is arcane wizardry
        // based on the specifics of george's weird
        // display and characters... don't fuck around w it
        let mut i = 0;
        for char_row in 0..29 {
            for char_col in 0..64 {
                let ascii = memory.read(0x6000 + i);
                i += 1;
                for row in 0..13 {
                    let byte = self.char_rom[ascii as usize + (row * 0x100)];
                    for i in (0..8).rev() {
                        let buffer_index = ((char_row) * 13 + (row)) * 512 + (char_col * 8 + i);
                        if (byte << i) & 0x80 == 0x80 {
                            self.buffer[buffer_index] = FG_COLOR;
                        } else {
                            self.buffer[buffer_index] = BG_COLOR;
                        }
                    }
                }
            }
        }

        let buffer = self.buffer.to_owned();

        let _ = self.window.send(buffer);
    }

    pub fn run(&mut self) {
        loop {
            let _ = self.interrupt.send(false);
            sleep(Duration::from_millis(16));
            self.draw();
            let _ = self.interrupt.send(true);
        }
    }
}
